@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 #    Copyright (C) 2013 Yahoo! Inc. All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -25,7 +23,7 @@ from taskflow.utils import async_utils
 from taskflow.utils import misc
 from taskflow.utils import threading_utils
 
-# Revert or execution events...
+# Execution and reversion events.
 EXECUTED = 'executed'
 REVERTED = 'reverted'
 
@@ -36,7 +34,7 @@ def _execute_task(task, arguments, progress_callback):
             result = task.execute(**arguments)
         except Exception:
             # NOTE(imelnikov): wrap current exception with Failure
-            # object and return it
+            # object and return it.
             result = misc.Failure()
     return (task, EXECUTED, result)
 
@@ -50,7 +48,7 @@ def _revert_task(task, arguments, result, failures, progress_callback):
             result = task.revert(**kwargs)
         except Exception:
             # NOTE(imelnikov): wrap current exception with Failure
-            # object and return it
+            # object and return it.
             result = misc.Failure()
     return (task, REVERTED, result)
 
@@ -65,42 +63,42 @@ class TaskExecutorBase(object):
     """
 
     @abc.abstractmethod
-    def execute_task(self, task, arguments, progress_callback=None):
+    def execute_task(self, task, task_uuid, arguments, progress_callback=None):
         """Schedules task execution."""
 
     @abc.abstractmethod
-    def revert_task(self, task, arguments, result, failures,
+    def revert_task(self, task, task_uuid, arguments, result, failures,
                     progress_callback=None):
-        """Schedules task reversion"""
+        """Schedules task reversion."""
 
     @abc.abstractmethod
     def wait_for_any(self, fs, timeout=None):
-        """Wait for futures returned by this executor to complete"""
+        """Wait for futures returned by this executor to complete."""
 
     def start(self):
-        """Prepare to execute tasks"""
+        """Prepare to execute tasks."""
         pass
 
     def stop(self):
-        """Finalize task executor"""
+        """Finalize task executor."""
         pass
 
 
 class SerialTaskExecutor(TaskExecutorBase):
     """Execute task one after another."""
 
-    def execute_task(self, task, arguments, progress_callback=None):
+    def execute_task(self, task, task_uuid, arguments, progress_callback=None):
         return async_utils.make_completed_future(
             _execute_task(task, arguments, progress_callback))
 
-    def revert_task(self, task, arguments, result, failures,
+    def revert_task(self, task, task_uuid, arguments, result, failures,
                     progress_callback=None):
         return async_utils.make_completed_future(
             _revert_task(task, arguments, result,
                          failures, progress_callback))
 
     def wait_for_any(self, fs, timeout=None):
-        # NOTE(imelnikov): this executor returns only done futures
+        # NOTE(imelnikov): this executor returns only done futures.
         return fs, []
 
 
@@ -115,11 +113,11 @@ class ParallelTaskExecutor(TaskExecutorBase):
         self._executor = executor
         self._own_executor = executor is None
 
-    def execute_task(self, task, arguments, progress_callback=None):
+    def execute_task(self, task, task_uuid, arguments, progress_callback=None):
         return self._executor.submit(
             _execute_task, task, arguments, progress_callback)
 
-    def revert_task(self, task, arguments, result, failures,
+    def revert_task(self, task, task_uuid, arguments, result, failures,
                     progress_callback=None):
         return self._executor.submit(
             _revert_task, task,
