@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 #    Copyright (C) 2013 Yahoo! Inc. All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -26,7 +24,7 @@ from taskflow.utils import misc
 
 
 class PrintingListener(base.LoggingBase):
-    """Writes the task and flow notifications messages to stdout or stderr"""
+    """Writes the task and flow notifications messages to stdout or stderr."""
     def __init__(self, engine,
                  task_listen_for=(misc.TransitionNotifier.ANY,),
                  flow_listen_for=(misc.TransitionNotifier.ANY,),
